# Bone Influence Tool by AzurieWolf
# Checks if any vertex has more than the set bone influences,
# keeping only the strongest per vertex.

import bpy
import bmesh
from bpy.types import Operator, Panel, PropertyGroup
from bpy.props import IntProperty, PointerProperty

# Tool Version
TOOL_VERSION = "1.0.1"


class BoneInfluenceProperties(PropertyGroup):
    max_bone_influences: IntProperty(
        name="Max Bone Influences",
        description="Maximum number of bone influences per vertex",
        default=4,
        min=1,
        max=10
    )

class OBJECT_OT_CheckBoneInfluences(Operator):
    bl_idname = "object.check_bone_influences"
    bl_label = "Check Bone Influences"
    bl_description = "Checks if any vertex has more than the set bone influences"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        obj = context.object
        max_influences = context.scene.bone_influence_props.max_bone_influences

        if bpy.context.object.mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT')
        
        if not obj or obj.type != 'MESH':
            self.report({'ERROR'}, "Select a valid mesh object.")
            return {'CANCELLED'}
        
        if not obj.vertex_groups:
            self.report({'ERROR'}, "No vertex groups found.")
            return {'CANCELLED'}
        
        mesh = obj.data
        bm = bmesh.new()
        bm.from_mesh(mesh)
        
        high_influence_verts = []
        for v in bm.verts:
            influence_count = sum(1 for g in obj.vertex_groups if any(vg.group == g.index for vg in obj.data.vertices[v.index].groups))
            if influence_count > max_influences:
                high_influence_verts.append(v.index)
        
        if high_influence_verts:
            self.report({'WARNING'}, f"{len(high_influence_verts)} vertices exceed {max_influences} bone influences!")
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='DESELECT')
            bpy.ops.object.mode_set(mode='OBJECT')
            for v_index in high_influence_verts:
                mesh.vertices[v_index].select = True
            bpy.ops.object.mode_set(mode='EDIT')
        else:
            self.report({'INFO'}, f"All vertices are within the {max_influences} bone influence limit.")
        
        return {'FINISHED'}

class OBJECT_OT_OptimizeBoneInfluences(Operator):
    bl_idname = "object.optimize_bone_influences"
    bl_label = "Optimize Bone Influences"
    bl_description = "Removes unnecessary bone influences, keeping only the strongest per vertex"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        obj = context.object
        max_influences = context.scene.bone_influence_props.max_bone_influences

        if bpy.context.object.mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT')
        
        if not obj or obj.type != 'MESH':
            self.report({'ERROR'}, "Select a valid mesh object.")
            return {'CANCELLED'}
        
        if not obj.vertex_groups:
            self.report({'ERROR'}, "No vertex groups found.")
            return {'CANCELLED'}
        
        for v in obj.data.vertices:
            if len(v.groups) > max_influences:
                # Convert to a list and sort it based on weight (strongest first)
                sorted_groups = sorted(v.groups, key=lambda g: obj.vertex_groups[g.group].weight(v.index), reverse=True)
                
                # Remove influences beyond the top 4
                for vg in sorted_groups[max_influences:]:
                    obj.vertex_groups[vg.group].remove([v.index])
        
        self.report({'INFO'}, f"Optimized bone influences. Only {max_influences} strongest per vertex kept.")
        return {'FINISHED'}

class OBJECT_PT_BoneInfluenceTools(Panel):
    bl_label = f"Bone Influence Tools v{TOOL_VERSION}"
    bl_idname = "OBJECT_PT_bone_influence_tools"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Tool"

    def draw(self, context):
        layout = self.layout
        props = context.scene.bone_influence_props

        layout.prop(props, "max_bone_influences")
        layout.operator("object.check_bone_influences", icon='MESH_DATA')
        layout.operator("object.optimize_bone_influences", icon='MODIFIER')

# Register Classes
def register():
    bpy.utils.register_class(BoneInfluenceProperties)
    bpy.utils.register_class(OBJECT_OT_CheckBoneInfluences)
    bpy.utils.register_class(OBJECT_OT_OptimizeBoneInfluences)
    bpy.utils.register_class(OBJECT_PT_BoneInfluenceTools)
    bpy.types.Scene.bone_influence_props = PointerProperty(type=BoneInfluenceProperties)

def unregister():
    bpy.utils.unregister_class(BoneInfluenceProperties)
    bpy.utils.unregister_class(OBJECT_OT_CheckBoneInfluences)
    bpy.utils.unregister_class(OBJECT_OT_OptimizeBoneInfluences)
    bpy.utils.unregister_class(OBJECT_PT_BoneInfluenceTools)
    del bpy.types.Scene.bone_influence_props

if __name__ == "__main__":
    register()